﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace SetCBLUserImage
{
    public class ProgramRecord : UserImageRecord
    {
        public byte MachineID { get; private set; }
        public byte DataTypeID { get; private set; }
        public string Name { get; private set; }
        public byte[] ProgramData { get; private set; }

        public ProgramRecord(byte machineID, byte dataTypeID, string name, byte[] data)
            : base(0x37AE, RecordType.Program, 0xB49D)
        {
            _Init(machineID, dataTypeID, name, data);
        }

        public ProgramRecord(ushort marker, RecordType type, ushort unknown1, byte[] data)
            : base(marker, type, unknown1)
        {
            int length = data[2] | (data[3] << 8);
            var name = ASCIIEncoding.ASCII.GetString(data, 4, length);

            byte machineID = data[0];
            byte[] progData;
            if (machineID >= 0x88) //HACK: if 68k series, use the first size word instead
            {
                int size = data[4 + length] | (data[4 + length + 1] << 8);
                progData = new byte[size];
                Array.Copy(data, 4 + length + 2, progData, 0, progData.Length);
            }
            else
            {
                int size = data[4 + length + 2] | (data[4 + length + 3] << 8);
                progData = new byte[size];
                Array.Copy(data, 4 + length + 4, progData, 0, progData.Length);
            }

            _Init(data[0], data[1], name, progData);
        }

        public override byte[] GetSaveData()
        {
            var stream = new MemoryStream();

            //Write machine ID, command ID, and name length
            stream.Write(new byte[] { MachineID, DataTypeID, 0x08, 0x00 }, 0, 4);

            //Write program name
            var bytes = new byte[8];
            var name = ASCIIEncoding.ASCII.GetBytes(Name, 0, Name.Length, bytes, 0);
            stream.Write(bytes, 0, bytes.Length);

            byte[] final;
            byte[] size;
            if (MachineID >= 0x88) //HACK: if 68k series, use one size word instead
            {
                //Calculate size as data + size bytes (and make sure it's even)
                var dataSize = ProgramData.Length + 2;
                if (dataSize % 2 == 1)
                {
                    dataSize++;
                }

                //Build length bytes
                final = new byte[dataSize];
                size = new byte[2];
                size[0] = (byte)(ProgramData.Length & 0xFF);
                size[1] = (byte)((ProgramData.Length >> 8) & 0xFF);
            }
            else
            {
                //Build length bytes
                final = new byte[ProgramData.Length + 4];
                size = new byte[4];
                size[0] = (byte)((ProgramData.Length + 2) & 0xFF);
                size[1] = (byte)(((ProgramData.Length + 2) >> 8) & 0xFF);
                size[2] = (byte)(ProgramData.Length & 0xFF);
                size[3] = (byte)((ProgramData.Length >> 8) & 0xFF);
            }

            Array.Copy(size, 0, final, 0, size.Length);

            //Write program data
            Array.Copy(ProgramData, 0, final, size.Length, ProgramData.Length);           
            stream.Write(final, 0, final.Length);

            return _WrapSaveData(stream.ToArray());
        }

        public override string ToString()
        {
            return Name;
        }

        private void _Init(byte machineID, byte dataTypeID, string name, byte[] data)
        {
            MachineID = machineID;
            DataTypeID = dataTypeID;

            Name = name;

            ProgramData = data;
        }
    }
}
