﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace SetCBLUserImage
{
    public class RealArrayRecord : UserImageRecord
    {
        public string Comment { get; private set; }
        public float[] Values { get; private set; }

        public RealArrayRecord(ushort marker, RecordType type, ushort unknown1, byte[] data)
            : base(marker, type, unknown1)
        {
            Comment = ASCIIEncoding.ASCII.GetString(data, 0, 0x1C).Trim('\0');
            
            int size = data[0x1C] | (data[0x1D] << 8) | (data[0x1E] << 16) | (data[0x1F] << 24);
            Values = new float[size];
            for (int i = 0; i < Values.Length; i++)
            {
                var bytes = new byte[4];
                Array.Copy(data, 0x20 + (i * 4), bytes, 0, bytes.Length);
                if (!BitConverter.IsLittleEndian)
                {
                    Array.Reverse(bytes);
                }

                Values[i] = BitConverter.ToSingle(bytes, 0);
            }
        }

        public override byte[] GetSaveData()
        {
            var stream = new MemoryStream();

            //Write space-padded comment string (and some more zeroes)
            var bytes = new byte[0x1C];
            var comment = ASCIIEncoding.ASCII.GetBytes(Comment.Substring(0, Math.Min(20, Comment.Length)));
            Array.Copy(comment, 0, bytes, 0, comment.Length);
            stream.Write(bytes, 0, bytes.Length);

            //Write number of items
            var size = new byte[4];
            size[0] = (byte)(Values.Length & 0xFF);
            size[1] = (byte)((Values.Length >> 8) & 0xFF);
            size[2] = (byte)((Values.Length >> 16) & 0xFF);
            size[3] = (byte)((Values.Length >> 24) & 0xFF);
            stream.Write(size, 0, size.Length);

            //Write the values
            foreach (var val in Values)
            {
                var data = BitConverter.GetBytes(val);
                if (!BitConverter.IsLittleEndian)
                {
                    Array.Reverse(bytes);
                }

                stream.Write(data, 0, data.Length);
            }

            return _WrapSaveData(stream.ToArray());
        }

        public override string ToString()
        {
            return Comment;
        }
    }
}
